"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const types_1 = require("@babel/types");
const never_1 = tslib_1.__importDefault(require("never"));
const findInterface_1 = tslib_1.__importDefault(require("./findInterface"));
const transformInterface = (path, node) => {
    if (types_1.isTSTypeReference(node) && types_1.isIdentifier(node.typeName)) {
        const { typeName: { name } } = node;
        const interfaceDeclaration = findInterface_1.default(path, name);
        if (interfaceDeclaration !== undefined) {
            interfaceDeclaration.body.body.forEach((node) => {
                transformTypeAnnotation(path, node.typeAnnotation, Boolean(node.optional));
            });
        }
    }
};
const transformTypeAnnotation = (path, typeAnnotation, isOptional) => {
    if (types_1.isTSTypeAnnotation(typeAnnotation) &&
        !isOptional) {
        if (types_1.isTSUnionType(typeAnnotation.typeAnnotation)) {
            if (!typeAnnotation.typeAnnotation.types.some(type => types_1.isTSUndefinedKeyword(type))) {
                typeAnnotation.typeAnnotation.types.push(types_1.tSUndefinedKeyword());
            }
            typeAnnotation.typeAnnotation.types.forEach(type => transformInterface(path, type));
        }
        else {
            transformInterface(path, typeAnnotation.typeAnnotation);
            typeAnnotation.typeAnnotation = types_1.tsUnionType([
                types_1.isTSFunctionType(typeAnnotation.typeAnnotation)
                    ? types_1.isTSParenthesizedType(typeAnnotation.typeAnnotation)
                        ? typeAnnotation.typeAnnotation
                        : types_1.tsParenthesizedType(typeAnnotation.typeAnnotation)
                    : typeAnnotation.typeAnnotation,
                types_1.tSUndefinedKeyword()
            ]);
        }
    }
};
const transformNode = (path, node) => {
    if (types_1.isArrowFunctionExpression(node) ||
        types_1.isFunctionDeclaration(node) ||
        types_1.isTSDeclareFunction(node)) {
        node.params.forEach((param) => {
            const { typeAnnotation } = param;
            transformTypeAnnotation(path, typeAnnotation, types_1.isIdentifier(param) && param.optional === true);
        });
    }
    else if (types_1.isTSInterfaceDeclaration(node)) {
        node.body.body.forEach(node => {
            if (types_1.isTSPropertySignature(node))
                node.optional = true;
        });
    }
};
const plugin = {
    visitor: {
        ExportDefaultDeclaration: path => {
            var _a;
            if (types_1.isIdentifier(path.node.declaration)) {
                const { name } = path.node.declaration;
                const { path: { node } } = (_a = path.scope.getBinding(name)) !== null && _a !== void 0 ? _a : never_1.default('No binding');
                if (types_1.isVariableDeclarator(node)) {
                    transformNode(path, node.init);
                }
            }
            else {
                transformNode(path, path.node.declaration);
            }
        },
        ExportDeclaration: path => {
            if (types_1.isExportNamedDeclaration(path.node)) {
                if (types_1.isVariableDeclaration(path.node.declaration)) {
                    transformNode(path, path.node.declaration.declarations[0].init);
                }
            }
        },
        ExportNamedDeclaration: path => {
            if (types_1.isTSDeclareFunction(path.node.declaration) ||
                types_1.isTSInterfaceDeclaration(path.node.declaration)) {
                transformNode(path, path.node.declaration);
            }
        }
    }
};
exports.default = plugin;
